from Products.CMFCore.utils import UniqueObject
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.permissions import AccessInactivePortalContent
from Products.CMFPlone.CatalogTool import CatalogTool
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import time
import urllib
import logging
import zope.interface
from zope.component.interfaces import ComponentLookupError
from zope.app import zapi

from zope.interface import implements

from Products.plonelucenecatalog.lib.brain import CPSBrain
from Products.plonelucenecatalog.lib.zcatalogquery import makeZCatalogQueries
from lib.objectregistry import LuceneObjectWrapper

from interfaces import IPloneLuceneCatalogTool

from nuxeo.lucene.catalog import LuceneCatalog

from Products.CMFCore.utils import _getAuthenticatedUser
from Products.CMFCore.utils import _checkPermission
from AccessControl.PermissionRole import rolesForPermissionOn

from Acquisition import aq_inner
from Acquisition import aq_parent
from Acquisition import aq_base
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import safe_callable
import re
from DateTime import DateTime


LOG = logging.getLogger("PloneLuceneCatalog")

class PloneLuceneCatalog(CatalogTool):
    """ MyTool  .... """

    zope.interface.implements(IPloneLuceneCatalogTool)

    id = 'portal_catalog'
    meta_type= 'PloneLuceneCatalog'
    plone_tool = 1

    server_url = 'http://localhost:9180'
    multilanguage_support = 1

    security = ClassSecurityInfo()

    def __init__(self):
        utility = LuceneCatalog(self.server_url)
        self._setOb('_catalog', utility)
        
        # BBB for CPS <= 3.4
        self.Indexes = self


    def  __setattr__(self, id, value):
        if id == 'server_url':
            if not value.startswith('http://'):
                value = 'http://' + value
            LOG.info("Update nuxeo.lucene.catalog properties")
            setattr(self.getCatalog(), id, value)
            self._refreshCatalogProxy()
        CatalogTool.__setattr__(self, id, value)

    def __url(self, ob):
        return '/'.join( ob.getPhysicalPath() )

    def __len__(self):
        return len(self.getCatalog())

    def __nonzero__(self):
        return True

    def _refreshCatalogProxy(self):
        """Refresh the cached proxy
        """
        # Activate persistency
        self.getCatalog()._p_changed = 1
        # Remove cached proxy
        self.getCatalog()._v_proxy = None

    def getCatalog(self):
        """ return the Catalog """
        return self._catalog

    def clean(self):
        #print "----CLEAN----"
        return self.getCatalog().clean()

    def catalog_object(self, object, uid, idxs=[], update_metadata=1,
                       pghandler=None):
        LOG.info("cat_catalog_object %s" % str(object))

        wf = getattr(self, 'portal_workflow', None)
        if wf is not None:
            vars = wf.getCatalogVariablesFor(object)
        else:
            vars = {}
            
        # Workaround for a Lucene problem:
        # In lucene, you can not get the data out from an unindexed or
        # unstored field. And since reindexing means unindexing and then
        # re-indexing a record, this means that if you don't pass the data
        # for unindexed or unstored fields, these will be cleared.
        # Therefore, when specifying which indexes to reindex, we here make
        # sure that all unindexed and unstored fields are included:
        if idxs:
            idxs = list(idxs)
            for idx in self.getCatalog().getFieldConfigurationsFor():
                if (idx.type.lower() in ('unindexed', 'unstored', 'sort') and
                    idx.name not in idxs):
                    idxs.append(idx.name)

        portal = aq_parent(aq_inner(self))
        w = LuceneObjectWrapper(vars, object, portal=portal)
        self.getCatalog().index(uid, w, idxs)

    security.declareProtected(ManagePortal, 'reindexObject')
    def reindexObject(self, object, idxs=[], update_metadata=1, uid=None):
        """ Update 'object' in catalog.
        """
        if uid is None:
            uid = self.__url(object)

        # Filter out invalid indexes will be done at nuxeo.lucene level.
        self.catalog_object(object, uid, idxs, update_metadata)
        
    # For plone compatibility we need to support idxs for indexObject:
    def indexObject(self, object, idxs=[]):
        """ Update 'object' in catalog.
        """
        # I don't know why Plone supports an idxs variable here. But it
        # does it in a buggy way. Uncomment the following lines to
        # reproduce the Plone bug:
        if idxs:
            for ix in ('allowedRolesAndUsers', 'expires', 'effective'):
                if ix not in idxs:
                    idxs.append(ix)
        self.reindexObject(object, idxs=idxs)

    def uncatalog_object(self, uid):
        self.getCatalog().unindex(uid)

    def uncatalogObject(self, uid):
        self.getCatalog().unindex(uid)

    def unindexObject(self, object):
        """ Remove 'object' from the catalog.
        o Permission:  Private (Python only)
        """
        LOG.info("unindexObject %s" % str(object))
        default_uid = self._CatalogTool__url(object)
        self.uncatalogObject(default_uid)

    def _search(self, REQUEST=None, **kw):
        queries = makeZCatalogQueries(self, REQUEST, **kw)
        
        # First query:
        sf = queries[0].getFieldsMap()
        qo =queries[0].getQueryOptions()
        results, nb_results = self.getCatalog().searchResults(
            search_fields=sf,
            options=qo,
        )
        if len(queries) > 1:
            # Some queries are made up of subqueries to be merged.
            uids = set([x['uid'] for x in results])
            for query in queries[1:]:
                sf = query.getFieldsMap()
                qo =query.getQueryOptions()
                res, nb_results = self.getCatalog().searchResults(
                    search_fields=sf,
                    options=qo,
                )
                for each in res:
                    if each['uid'] not in uids:
                        uids.add(each['uid'])
                        results += (each,)

        brains = []
        for mapping in results:
            one = CPSBrain(mapping).__of__(self)
            setattr(one, 'out_of', nb_results)
            brains.append(one)
        return brains

    def searchResults(self, k=None, REQUEST=None, **kw):
        """Searching with CPS security indexes.
        """
        kw = kw.copy() 
        if kw != {} and k:
            kw.update(k)
        elif kw == {} and k:
            kw = k

        user = _getAuthenticatedUser(self)
        kw[ 'allowedRolesAndUsers' ] = self._listAllowedRolesAndUsers(user)
        
        # Do all the checks to make sure a user that shouldn't see inactive 
        # content doesn't see it unless he specifically asks for it, but 
        # still can restrict searches on effective and expired if desired:
        now = DateTime()
        show_inactive = (_checkPermission(AccessInactivePortalContent, self) or
                         kw.get('show_inactive', False))
        
        if not show_inactive:
            # Make sure inactive are not shown.
            if kw.has_key('effective'):
                if isinstance(kw['effective'], dict):
                    effective_date = kw['effective']['query']
                    kw['effective']['range'] = 'max'
                else:
                    # Make it a dict:
                    effective_date = kw['effective']
                    kw['effective'] = {'query': effective_date,
                                       'range': 'max'}
                if effective_date > now:
                    # Trying to show things that are not active, when they
                    # shouldn't be shown.
                    del kw['effective']

            if not kw.has_key('effective'):
                kw['effective'] = {'query': now,
                                   'range': 'max'}

            if kw.has_key('expires'):
                if isinstance(kw['expires'], dict):
                    expires_date = kw['expires']['query']
                    kw['expires']['range'] = 'min'
                else:
                    expires_date = kw['expires']
                    kw['expires'] = {'query': expires_date,
                                     'range': 'min'}
                if expires_date < now:
                    # Trying to show things that are not active, when they
                    # shouldn't be shown.
                    del kw['expires']
                    
            if not kw.has_key('expires'):
                kw['expires'] = {'query': now,
                                 'range': 'min'}
            
        return self._search(REQUEST, **kw)

    # Override CMFCore.CatalogTool alias
    __call__ = searchResults

    def unrestrictedSearchResults(self, REQUEST=None, **kw):
        """Calls ZCatalog.searchResults() without any CMF-specific processing.

        o Permission:  Private (Python only)
        """
        return self._search(REQUEST, **kw)

    def indexes(self):
        b=self.getFieldConfs()
        indexes = [y.get('name') for y in b]
        return indexes


    security.declareProtected(ManagePortal, 'addField')
    def addField(self, **kw):
        self.getCatalog().addFieldFor(iface=None, **kw)
        return None

    security.declareProtected(ManagePortal, 'addColumn')
    def addColumn(self, name, default_value=None):
        return self.getCatalog().addColumn(name, default_value)

    security.declareProtected(ManagePortal, 'getFieldTypes')
    def getFieldTypes(self):
        # XXX : hardcoded for now and not complete
        fields =  (
            'Keyword',
            'MultiKeyword', # XXX will disappear in the future.
            'Boolean', # XXX will disappear in the future.
            'Text',
            'UnStored',
            'UnIndexed',
            'Date',
            'Path',
            'Sort',
            )
        return fields

    security.declareProtected(ManagePortal, 'getFieldConfs')
    def getFieldConfs(self):
        # It applies globally for now. nuxeo.lucene can discriminate
        # by iface though. 
        cpsres = ()
        fieldconfs = self.getCatalog().getFieldConfigurationsFor()
        for each in fieldconfs:
            cpsres += (
                {'name' : each.name,
                 'attribute' : each.attribute,
                 'type' : each.type,
                 'analyzer' : each.analyzer,
                 },)
        return cpsres


    def uniqueValuesFor(self,name):
        return self._catalog.getFieldTerms(name)

    #
    # ZMI
    #

    # The server_url properties are here only to display the values in ZMI.
    # The actual used properties are on the nuxeo.lucene catalog
    # utility
    _properties = CatalogTool._properties + \
                  ({'id':'server_url',
                    'type':'string',
                    'mode':'w',
                    'label':'xml-rpc server URL',
                    },
                    {'id':'multilanguage_support',
                    'type':'boolean',
                    'mode':'w',
                    'label':'Multi-language support',
                    },                   )

    manage_options = (CatalogTool.manage_options[2],
                      { 'label' : 'Manage XML-RPC Server',
                        'action' : 'manage_advancedForm',
                        },
                      { 'label' : 'Schema',
                        'action' : 'manage_catalogSchema',
                        },
                      { 'label' : 'Fields',
                        'action' : 'manage_catalogFields',
                        },
                      )

    security.declareProtected(ManagePortal, 'manage_advancedForm')
    manage_advancedForm = PageTemplateFile(
        'zmi/manage_advancedForm.pt', globals())

    security.declareProtected(ManagePortal, 'manage_catalogFields')
    manage_catalogFields = PageTemplateFile(
        'zmi/manage_catalogFields.pt', globals())

    security.declareProtected(ManagePortal, 'manage_catalogSchema')
    manage_catalogSchema = PageTemplateFile(
        'zmi/manage_catalogSchema.pt', globals())

    security.declareProtected(ManagePortal, 'manage_reindexAll')
    def manage_reindexAll(self, idxs=(), REQUEST=None, RESPONSE=None, URL1=None):
        """Reindex  an existing complete CPS instance with content.

        It checks all the CPS proxies from the proxies tool.
        """
        elapse = time.time()
        c_elapse = time.clock()

        self.manage_catalogClear()
        portal = aq_parent(aq_inner(self))
        portal.ZopeFindAndApply(portal, search_sub=True,
                                apply_func=reindexContentObject)

        elapse = time.time() - elapse
        c_elapse = time.clock() - c_elapse

        if REQUEST and RESPONSE:
            RESPONSE.redirect(
                URL1 +
                '/manage_advancedForm?manage_tabs_message=' +
                urllib.quote('Catalog Cleared and Recreated \n'
                             'Total time: %s\n'
                             'Total CPU time: %s' % (`elapse`, `c_elapse`)))

    # Plone API
    clearFindAndRebuild = manage_reindexAll

    security.declareProtected(ManagePortal, 'manage_reindexSelectedFields')
    def manage_reindexSelectedFields(self, fields=(), REQUEST=None):
        """Reindex selected fields.
        """

        if fields:
            self.manage_reindexAll(idxs=fields, REQUEST=REQUEST)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(
                self.absolute_url() + '/manage_catalogFields')

    security.declareProtected(ManagePortal, 'manage_optimize')
    def manage_optimize(self, REQUEST=None, RESPONSE=None, URL1=None):
        """Optimier the indexes store
        """
        self.getCatalog().optimize()
        if REQUEST and RESPONSE:
            RESPONSE.redirect(
                URL1 +
                '/manage_advancedForm?manage_tabs_message=Catalog Optimized')



    security.declareProtected(ManagePortal, 'manage_catalogClear')
    def manage_catalogClear(self, REQUEST=None, RESPONSE=None, URL1=None):
        """Clean the indexes store.
        """
        self.clean()
        if REQUEST and RESPONSE:
            RESPONSE.redirect(
                URL1 +
                '/manage_advancedForm?manage_tabs_message=Catalog Cleaned')


InitializeClass(PloneLuceneCatalog)

# Utility function for calling reindexObject from ZopeFindAndApply
def reindexContentObject(obj, *args):
    """A method which reindexes an object if it is content.
       The ZopeFindAndApply method expects a function that takes both an
       object and a path as positional parameters."""
    if base_hasattr(obj, 'reindexObject') and \
            safe_callable(obj.reindexObject):
        try:
            obj.reindexObject()
        except TypeError:
            # Catalogs have this method as well, but they take
            # different args, and will fail
            pass
