import re
import logging
from zope.interface import implements
from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectWrapper, ExtensibleIndexableObjectRegistry
from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectRegistry
from Products.CMFCore.interfaces.portal_catalog \
        import IndexableObjectWrapper as z2IIndexableObjectWrapper
from Products.CMFCore.interfaces import IIndexableObjectWrapper

from Products.CMFCore.utils import _getAuthenticatedUser, getToolByName
from AccessControl.PermissionRole import rolesForPermissionOn

from Acquisition import aq_inner
from Acquisition import aq_parent
from Acquisition import aq_base
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import safe_callable
from DateTime import DateTime

from Products.CMFPlone.CatalogTool import registerIndexableAttribute

LOG = logging.getLogger("PloneLuceneCatalog")


class LuceneObjectWrapper(ExtensibleIndexableObjectWrapper):
    
    def __getattr__(self, name):
        res = super(LuceneObjectWrapper, self).__getattr__(name)
        if callable(res):
            res = res()
        if isinstance(res, DateTime):
            return res.ISO()
        return res

def zero_fill(matchobj):
    return matchobj.group().zfill(8)

num_sort_regex = re.compile('\d+')
def sortable_title(obj, portal, **kwargs):
    """ Helper method for to provide FieldIndex for Title.

    >>> from Products.CMFPlone.CatalogTool import sortable_title
    
    >>> self.folder.setTitle('Plone42 _foo')
    >>> sortable_title(self.folder, self.portal)
    'plone00000042 _foo'
    """
    def_charset = portal.plone_utils.getSiteEncoding()
    title = getattr(obj, 'Title', None)
    if title is not None:
        if safe_callable(title):
            title = title()
        if isinstance(title, basestring):
            sortabletitle = title.lower().strip()
            # Replace numbers with zero filled numbers
            sortabletitle = num_sort_regex.sub(zero_fill, sortabletitle)
            # Truncate to prevent bloat
            for charset in [def_charset, 'latin-1', 'utf-8']:
                try:
                    sortabletitle = unicode(sortabletitle, charset)[:30]
                    sortabletitle = sortabletitle.encode(def_charset or 'utf-8')
                    break
                except UnicodeError:
                    pass
                except TypeError:
                    # If we get a TypeError if we already have a unicode string
                    sortabletitle = sortabletitle[:30]
                    break
            return sortabletitle
    return ''
registerIndexableAttribute('sortable_title', sortable_title)


def getRemoteUrl(obj, portal, **kwargs):
    """Is this the default page in its folder
    """
    hasa = getattr(obj,'getRemoteUrl',False)
    if hasa != False:
        hasa = obj.getRemoteUrl()
    return hasa
registerIndexableAttribute('getRemoteUrl', getRemoteUrl)



def allowedRolesAndUsers(obj, portal, **kwargs):
    """Return a list of roles and users with View permission.

    Used by PortalCatalog to filter out items you're not allowed to see.
    """
    allowed = {}
    for r in rolesForPermissionOn('View', obj):
        allowed[r] = 1
    try:
        localroles = portal.acl_users._getAllLocalRoles(obj)
    except AttributeError:
        localroles = _mergedLocalRoles(obj)
    for user, roles in localroles.items():
        for role in roles:
            if allowed.has_key(role): 
                allowed['user:' + user] = 1
    if allowed.has_key('Owner'):
        del allowed['Owner']
    return list(allowed.keys())
registerIndexableAttribute('allowedRolesAndUsers', allowedRolesAndUsers)



def is_default_page(obj, portal, **kwargs):
    """Is this the default page in its folder
    """
    ptool = portal.plone_utils
    return ptool.isDefaultPage(obj)
registerIndexableAttribute('is_default_page', is_default_page)



def expires(obj, portal, **kwargs):
    """ Look what to do with expires """
    expires = obj.expires()
    if isinstance(expires,DateTime):
        expires = expires.ISO()
    return expires 
registerIndexableAttribute('expires', expires)
    
def relative_path_depth(self, portal, **kwargs):
    """This is used to produce an index
    return the path depth relative to the portal."""
    utool = getToolByName(portal, 'portal_url', None)
    ret = -1
    if utool:
	# broken object can't aquire portal_url
	rpath = utool.getRelativeContentURL(self)
	if rpath:
	    ret = rpath.count('/')+1
    return ret
registerIndexableAttribute('relative_path_depth', relative_path_depth)
