import logging
from string import lower 
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName

logger = logging.getLogger('PloneLuceneCatalog.zcatalgquery')

def makeDepthQueryValue(cat, path, depth):
    utool = getToolByName(cat, 'portal_url')
    ppath = utool.getPortalPath()
    start = (path[len(ppath):]).count('/')+1
    if depth < 1:
        depth_value = depth
    else:
        depth_range = range(start, start+depth)
        depth_value = [str(x) for x in depth_range]
    return depth_value

def makeZCatalogQueries(cat, REQUEST, **kw):
    zqueries = []
    for key, query in kw.items():
        if lower(key) == 'path':
            # Path queries in Plone can be very complex. They often
            # require several separate Lucene queries to be merged.
            
            if not isinstance(query, dict):
                query = {'query': query}
                
            navtree = query.get('navtree', False)
            depth = query.get('depth', None)
            path = query['query']
            if isinstance(path, tuple):
                path = '/'.join(path)
            if navtree:
                # Navtrees can be just breadcrumbs, or they can be
                # breadcrumbs + path search. That case needs to be handled
                # with separate queries:
                if depth == 0:
                    # A breadcrumb. We do that specially for speed so we can
                    # do it with only one query:
                    del kw['path']
                    uid_value = []
                    while '/' in path:
                        uid_value.append(path)
                        path = path[:path.rfind('/')]
                    kw['uid'] = uid_value
                    depth = None
                else:
                    if depth is None:
                        # Default for navtrees is 1. Default for paths is "no
                        # end". That's Logic! (Or something).
                        depth = 1
                        kw['path']['depth'] = 1

                    # We should make one tree per object in the breadcrumb path
                    # so we need to have a separate query for each level:
                    tree_kw = kw.copy()
                    path_kw = kw['path'].copy()
                    parent_path = path[:path.rfind('/')]
                    while '/' in parent_path:
                        path_kw['query'] = parent_path
                        tree_kw['path'] = path_kw
                        depthq = makeDepthQueryValue(cat, parent_path, depth)
                        tree_kw['relative_path_depth'] = depthq
                        zqueries.append(ZCatalogQuery(cat, REQUEST, **tree_kw))
                        parent_path = parent_path[:parent_path.rfind('/')]
                        
            if depth is not None:
                # Add a depth limit:
                depthq = makeDepthQueryValue(cat, path, depth)
                kw['relative_path_depth'] = depthq
                    
    zqueries.append(ZCatalogQuery(cat, REQUEST, **kw))
    return zqueries


class ZCatalogQuery(object):
    """ Wraps a ZCatalogQuery into a Lucene query"""
    
    def __init__(self, cat, REQUEST, **kw):

        self.fields = ()
        self.options = {}

        fields = cat.getCatalog().getFieldNamesFor()
        
        for kw, value in kw.items():
            if kw in ('sort_on','sort_order','sort_limit'):
                self.options[kw.replace('_','-')]=value
                continue
            if kw not in fields:
                continue
                
            field_conf={}
            field_conf={'id'   : kw, 
                        'value': value }
        
            if isinstance(value, dict):
                if 'query' in value.keys() and 'range' in value.keys():
                    field_conf['usage'] = 'range:' + value['range']                    
                    
                value = value['query']
                    
            ### Handle DateTimes:
            if isinstance(value, DateTime):
                value = value.ISO()
            elif isinstance(value, list) or isinstance(value, tuple):
                new_val = []
                for v in value:
                    if isinstance(v, DateTime):
                        new_val.append(v.ISO())
                    else:
                        new_val.append(v)
                value = new_val
                
            field_conf['value'] = value

            self.fields += (field_conf,)

    def getFieldsMap(self):
        return self.fields

    def getQueryOptions(self):
        return self.options
