# -*- coding: UTF-8 -*-
from Products.plonelucenecatalog.lib.zcatalogquery import ZCatalogQuery 
from DateTime import DateTime
import unittest 

kws={'review_state': ('published',)}

class FakeCatalog(object):
    def __init__(self, field_ids=()):
        self._field_ids = field_ids
    def getCatalog(self):
        return self
    def getFieldNamesFor(self):
        return self._field_ids

class SetupText(unittest.TestCase):                   
    def testSetup(self):                        
        """Init with no values"""  
        zc = ZCatalogQuery(FakeCatalog,None)
        self.assertEqual(zc.getFieldsMap(), ()) 

class ZQueryTestCase(unittest.TestCase):
    def test_simple_query(self):
        cat = FakeCatalog(field_ids=('Title', 'Description'))
        zq = ZCatalogQuery(cat, REQUEST=None, Title='foo', Description='Bar')
        self.assertEqual(zq.getFieldsMap(),
                         ({'id': 'Description', 'value': 'Bar'},
                          {'id': 'Title', 'value': 'foo'})
                         )
        self.assertEqual(zq.getQueryOptions(), {})

    def test_list_query(self):
        cat = FakeCatalog(field_ids=('Title', 'Description'))
        zq = ZCatalogQuery(cat, REQUEST=None, Title=('foo',), Description=('Bar',))
        self.assertEqual(zq.getFieldsMap(),
                         ({'id': 'Description', 'value': ['Bar']},
                          {'id': 'Title', 'value': ['foo']})
                         )
        self.assertEqual(zq.getQueryOptions(), {})

    def test_simple_date(self):
        cat = FakeCatalog(field_ids=('Date',))
        cdate = DateTime()
        zq = ZCatalogQuery(cat, REQUEST=None, Date=cdate)
        self.assertEqual(zq.getFieldsMap(),
                         ({'id': 'Date', 'value': cdate.ISO()},
                         ))

    def test_range_date_min(self):
        cat = FakeCatalog(field_ids=('Date',))
        cdate = DateTime()
        zq = ZCatalogQuery(cat, REQUEST=None, Date={'query':cdate, 'range': 'min'})
        self.assertEqual(zq.getFieldsMap(),
                         ({'usage': 'range:min', 'id': 'Date', 'value': cdate.ISO()},),
                         )

    def test_range_date_max(self):
        cat = FakeCatalog(field_ids=('Date',))
        cdate = DateTime()
        zq = ZCatalogQuery(cat, REQUEST=None, Date={'query':cdate, 'range': 'max'})
        self.assertEqual(zq.getFieldsMap(),
                         ({'usage': 'range:max', 'id': 'Date', 'value': cdate.ISO()},),
                         )

    def test_range_date_max_p(self):
        cat = FakeCatalog(field_ids=('Date',))
        cdate = DateTime()
        zq = ZCatalogQuery(cat, REQUEST=None, end = {'query': DateTime('2007/01/01'), 'range': 'min'})
        self.assertEqual(zq.getFieldsMap(),
                         ({'usage': 'range:min', 'id': 'end', 'value': DateTime('2007/01/01').ISO()},),
                         )
    def test_range_date_minmax(self):
        cat = FakeCatalog(field_ids=('Date',))
        cdate1 = DateTime()
        cdate2 = DateTime()
        zq = ZCatalogQuery(cat, REQUEST=None, Date={'query':[cdate1, cdate2],
                                                    'range': 'min:max'})
        self.assertEqual(zq.getFieldsMap(),
                         ({'usage': 'range:min:max', 'id': 'Date',
                           'value': [cdate1.ISO(), cdate2.ISO()]},),
                         )

    def test_path(self):
        cat = FakeCatalog(field_ids=('path',))
        zq = ZCatalogQuery(cat, REQUEST=None, path= {'query': '/LUC', 'depth': 1})
        self.assertEqual(zq.getFieldsMap(),
                         ({'value': '/LUC', 'id': 'path', 'usage':'depth:1'},))

    def test_sorton_query(self):
        cat = FakeCatalog(field_ids=(''))
        zq = ZCatalogQuery(cat, REQUEST=None, sort_on='foo' )
        self.assertEqual(zq.getFieldsMap(),())
        self.assertEqual(zq.getQueryOptions(), {'sort-on': 'foo'})

#if __name__ == "__main__": 
#    unittest.main()   

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ZQueryTestCase))
    return suite


