#
# Skeleton ZopeTestCase
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.ZCatalog.ZCatalog import ZCatalog
from OFS.SimpleItem import SimpleItem
from Products.PloneTestCase import PloneTestCase 
from time import sleep
import transaction
from DateTime import DateTime


class TestLuceneIndexes(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        self.catalog = self.portal.portal_catalog
        self.catalog.manage_catalogClear()
        self.p = self.portal

    def testId(self):
        self.folder.invokeFactory('Document','gustav')
        sleep(0.1)
        self.assertEqual(len(self.catalog(id='gustav')), 1)
        self.assertEqual(len(self.catalog(id='Nongustav')), 0)

    def testPortalType(self):
        self.folder.invokeFactory('Document','pttest')
        self.folder.invokeFactory('Folder','foldertest')
        sleep(0.1)
        self.assertEqual(len(self.catalog(portal_type=['Document',])), 1)
        self.assertEqual(len(self.catalog(portal_type=['Document','Folder'])), 2)

    def testStringSearchFilter(self):
        self.folder.invokeFactory('Folder','foldertest')
        self.folder.invokeFactory('Document','pttest')
        sleep(0.1)
        self.assertEqual(len(self.catalog(portal_type='Document')), 1)
    
    def testListSearchFilter(self):
        self.folder.invokeFactory('Document','pttest')
        sleep(0.1)
        self.assertEqual(len(self.catalog(portal_type=('Document'))), 1)

    def testListSearchFiltersR(self):
        self.folder.invokeFactory('Document','pttest')
        sleep(0.1)
        kw={'review_state': ('visible')}
        self.assertEqual(len(self.catalog(kw)), 1)

    def testZPath(self):
        self.catalog.manage_catalogClear()
        self.folder.invokeFactory('Folder','folderA')
        self.folder.invokeFactory('Folder','folderB')
        self.folder.invokeFactory('Folder','folderC')
        sleep(0.4)
        self.assertEqual(len(self.catalog(portal_type=['Folder',])), 3)
        pathsearch = {'query':"/plone"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 3)
        pathsearch = {'query':"/plone/Members/test_user_1_"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 3)
        pathsearch = {'query':"/plone/Members/test_user_1_/folderA"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 1)

        pathsearch = {'query':"/nogoodpath"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 0)

        self.folder.folderA.invokeFactory('Folder','folderAA')
        self.folder.folderA.folderAA.invokeFactory('Folder','folderAAA')
        self.folder.folderB.invokeFactory('Folder','folderA')
        sleep(0.4)
        self.assertEqual(len(self.catalog(portal_type=['Folder',])), 6)
        pathsearch = {'query':"/plone"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 6)
        pathsearch = {'query':"/plone/Members/test_user_1_"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 6)
        pathsearch = {'query':"/plone/Members/test_user_1_/folderA"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 3)
        pathsearch = {'query':"/plone/Members/test_user_1_/folderA/folderAA"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 2)
        pathsearch = {'query':"/plone/Members/test_user_1_/folderA/folderAA/folderAAA"}
        self.assertEqual(len(self.catalog(path=pathsearch)), 1)


    def testExpires(self):
        self.folder.invokeFactory('Document','document1')
        sleep(0.1)
        expDate = DateTime('2499/12/31')
        self.assertEqual(len(self.catalog(id='document1')), 1)
        obj = self.catalog(id='document1')[0]
        self.assertEqual(obj.expires, expDate)

    def testBooleanField(self):
        """ need little custom on z3 lucenecatalog """
        self.folder.invokeFactory('Document','document1')
        sleep(0.1)
        self.assertEqual(len(self.catalog(id='document1')), 1)
        obj = self.catalog(id='document1')[0]
        self.assertEqual(obj.is_folderish, False)

    def beforeTearDown(self):
        self.catalog.manage_catalogClear()
        pass

    def beforeClose(self):
        pass 
        #get_transaction().commit()


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLuceneIndexes))
    return suite

if __name__ == '__main__':
    framework()

