#
# These tests are adapted from Plones catalog tool tests.
#

import os, sys, time
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFPlone.tests import PloneTestCase

from Acquisition import aq_base
from Globals import REPLACEABLE
from DateTime import DateTime
from Products.CMFCore.permissions import AccessInactivePortalContent
import transaction

from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectWrapper
from Products.CMFPlone.CatalogTool import _eioRegistry

from Products.CMFPlone.CatalogTool import is_folderish
from Products.CMFPlone.tests import dummy

portal_name = PloneTestCase.portal_name
default_user  = PloneTestCase.default_user

user2  = 'u2'
group2 = 'g2'

base_content = ['Members', 'events', 'news', 'previous',
                default_user, 'front-page', 'doc']

path_content = ['/a1',
                '/a1/a2',
                '/a1/a2/a3',
                '/a1/a2/a3/a4',
                '/a1/a2/a3/b4',
                '/a1/a2/a3/c4',
                '/a1/a2/b3',
                '/a1/a2/b3/a4',
                '/a1/a2/b3/b4',
                '/a1/b2',
                '/a1/b2/a3',
                '/a1/b2/a3/a4',
                '/a1/b2/a3/b4',
                '/a1/b2/b3',
                '/b1',
                '/b1/a2',
                '/b1/a2/a3',
            ]
    
class TestCatalogPaths(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        self.catalog = self.portal.portal_catalog
        self.catalog._catalog._lucene_sync = True
        self.setRoles(('Manager',))
        self.catalog.manage_catalogClear()
        for url in path_content:
            path = url.split('/')
            ob = self.portal
            for id in path:
                if not id:
                    continue
                subob = getattr(ob, id, None)
                if subob is None:
                    ob.invokeFactory('Folder', id)
                    subob = getattr(ob, id, None)
                ob = subob

    def assertResults(self, result, expect):
        # Verifies ids of catalog results against expected ids
        lhs = [r.uid for r in result]
        lhs.sort()
        rhs = ['/%s%s' % (PloneTestCase.portal_name, x) for x in expect]
        rhs.sort()
        self.assertEqual(lhs, rhs)

    def test_search_all(self):
        res = self.catalog.searchResults()
        self.assertResults(res, path_content)

    def test_path_tree(self):
        res = self.catalog.searchResults(path='/plone/a1')
        self.assertResults(res, path_content[:14])

    def test_path_tree_depth0(self):
        res = self.catalog.searchResults(path={'query': '/plone/a1', 'depth': 0})
        self.assertResults(res, [])
        
    def test_path_tree_depth1(self):
        res = self.catalog.searchResults(path={'query': '/plone/a1', 'depth': 1})
        expected = ['/a1/a2',
                    '/a1/b2',
                ]
        self.assertResults(res, expected)

    def test_path_tree_depth2(self):
        res = self.catalog.searchResults(path={'query': '/plone/a1', 'depth': 2})
        expected = ['/a1/a2',
                    '/a1/a2/a3',
                    '/a1/a2/b3',
                    '/a1/b2',
                    '/a1/b2/a3',
                    '/a1/b2/b3',
                ]
        self.assertResults(res, expected)

    def test_path_navtree(self):
        res = self.catalog.searchResults(path={'query': '/plone/a1/a2', 'navtree': 1})
        expected = ['/a1',
                    '/a1/a2',
                    '/a1/a2/a3',
                    '/a1/a2/b3',
                    '/a1/b2',
                    '/b1',
                ]
        self.assertResults(res, expected)

    def test_path_navtree_depth2(self):
        res = self.catalog.searchResults(path={'query': '/plone/a1', 
                                               'navtree': 1,
                                               'depth': 2})
        expected = ['/a1',
                    '/a1/a2',
                    '/a1/a2/a3',
                    '/a1/a2/b3',
                    '/a1/b2',
                    '/a1/b2/a3',
                    '/a1/b2/b3',
                    '/b1',
                    '/b1/a2',
                ]
        self.assertResults(res, expected)

    def test_path_navtree_breadcrumb(self):
        res = self.catalog.searchResults(path={'query': '/plone/a1/a2', 
                                               'navtree': 1,
                                               'depth': 0})
        expected = ['/a1',
                    '/a1/a2',
                ]
        self.assertResults(res, expected)
        

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCatalogPaths))
    return suite

if __name__ == '__main__':
    framework()
